@@ -22,32 +22,33 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <cairo/cairo.h>
 #include "mode.h"
 
-static guint16 _lerp( gdouble t, guint16 a, guint16 b )
+static gdouble _lerp( gdouble t, gdouble a, gdouble b )
 {
-	return (guint16) (a + t * (b - a));
+	return (gdouble) (a + t * (b - a));
 }
 
-static void mix_colors( gdouble ratio, GdkColor *color1, GdkColor *color2, GdkGC *target )
+static void mix_colors( gdouble ratio, GdkRGBA *color1, GdkRGBA *color2, cairo_t *target )
 {
-	GdkColor color;
+	GdkRGBA color;
 	color.red = _lerp (ratio, color1->red, color2->red);
 	color.green = _lerp (ratio, color1->green, color2->green);
 	color.blue = _lerp (ratio, color1->blue, color2->blue);
-	gdk_gc_set_rgb_fg_color( target, &color );
+	color.alpha = 1.0;
+	gdk_cairo_set_source_rgba( target, &color );
 }
 
-void draw_graph_normal( CPUGraph *base, GtkWidget *da, gint w, gint h )
+void draw_graph_normal( CPUGraph *base, cairo_t *cr, gint w, gint h )
 {
 	gint x, y;
 	gint usage;
 	gdouble t;
 	gint tmp;
-	GdkGC *fg1 = gdk_gc_new( da->window );
 
 	if( base->color_mode == 0 )
-		gdk_gc_set_rgb_fg_color( fg1, &base->colors[1] );
+		gdk_cairo_set_source_rgba( cr, &base->colors[1] );
 
 	for( x = 0; x < w; x++ )
 	{
@@ -57,7 +58,11 @@ void draw_graph_normal( CPUGraph *base, GtkWidget *da, gint w, gint h )
 
 		if( base->color_mode == 0 )
 		{
-			gdk_draw_line( da->window, fg1, x, h-usage, x, h-1 );
+			/* draw line */
+			cairo_set_line_cap( cr, CAIRO_LINE_CAP_SQUARE );
+			cairo_move_to( cr, x, h - usage );
+			cairo_line_to( cr, x, h - 1 );
+			cairo_stroke( cr );
 		}
 		else
 		{
@@ -67,15 +72,18 @@ void draw_graph_normal( CPUGraph *base, GtkWidget *da, gint w, gint h )
 				t = (base->color_mode == 1) ?
 					(tmp / (gdouble) (h)) :
 					(tmp / (gdouble) (usage));
-				mix_colors( t, &base->colors[1], &base->colors[2], fg1);
-				gdk_draw_point( da->window, fg1, x, y );
+				mix_colors( t, &base->colors[1], &base->colors[2], cr );
+				/* draw point */
+				cairo_set_line_cap( cr, CAIRO_LINE_CAP_ROUND );
+				cairo_move_to( cr, x, y );
+				cairo_close_path (cr);
+				cairo_stroke( cr );
 			}
 		}
 	}
-	g_object_unref( fg1 );
 }
 
-void draw_graph_LED( CPUGraph *base, GtkWidget *da, gint w, gint h )
+void draw_graph_LED( CPUGraph *base, cairo_t *cr, gint w, gint h )
 {
 	gint nrx = (w + 1) / 3;
 	gint nry = (h + 1) / 2;
@@ -83,11 +91,6 @@ void draw_graph_LED( CPUGraph *base, GtkWidget *da, gint w, gint h )
 	gint idx;
 	gint limit;
 
-	GdkGC *fg1 = gdk_gc_new( da->window );
-	GdkGC *fg2 = gdk_gc_new( da->window );
-	gdk_gc_set_rgb_fg_color( fg1, &base->colors[1] );
-	gdk_gc_set_rgb_fg_color( fg2, &base->colors[2] );
-
 	for( x = 0; x * 3 < w; x++ )
 	{
 		idx = nrx-x;
@@ -99,27 +102,31 @@ void draw_graph_LED( CPUGraph *base, GtkWidget *da, gint w, gint h )
 				gdouble t = (base->color_mode == 1) ?
 				           (y / (gdouble)nry) :
 				           (y / (gdouble)limit);
-				mix_colors( t, &base->colors[3], &base->colors[2], fg2);
+				mix_colors( t, &base->colors[3], &base->colors[2], cr );
 			}
-			gdk_draw_rectangle (da->window, y >= limit ? fg1 : fg2, TRUE, x * 3, y * 2, 2, 1);
+			else
+			{
+				gdk_cairo_set_source_rgba( cr, y >= limit ? &base->colors[1] : &base->colors[2] );
+			}
+			/* draw rectangle */
+			cairo_rectangle( cr, x * 3, y * 2, 2, 1 );
+			cairo_fill( cr );
 		}
 	}
-	g_object_unref( fg1 );
-	g_object_unref( fg2 );
 }
 
-void draw_graph_no_history( CPUGraph *base, GtkWidget *da, gint w, gint h )
+void draw_graph_no_history( CPUGraph *base, cairo_t *cr, gint w, gint h )
 {
 	gint y;
 	gint usage = h * base->history[0] / CPU_SCALE;
 	gint tmp = 0;
 	gdouble t;
-	GdkGC *fg1 = gdk_gc_new( da->window );
 
 	if( base->color_mode == 0 )
 	{
-		gdk_gc_set_rgb_fg_color( fg1, &base->colors[1] );
-		gdk_draw_rectangle( da->window, fg1, TRUE, 0, h-usage, w, usage );
+		gdk_cairo_set_source_rgba( cr, &base->colors[1] );
+		cairo_rectangle( cr, 0, h - usage, w, usage );
+		cairo_fill( cr );
 	}
 	else
 	{
@@ -128,12 +135,15 @@ void draw_graph_no_history( CPUGraph *base, GtkWidget *da, gint w, gint h )
 			t = (base->color_mode == 1) ?
 				(tmp / (gdouble) (h)) :
 				(tmp / (gdouble) (usage));
-			mix_colors( t, &base->colors[1], &base->colors[2], fg1 );
+			mix_colors( t, &base->colors[1], &base->colors[2], cr );
 			tmp++;
-			gdk_draw_line( da->window, fg1, 0, y, w-1, y );
+			/* draw line */
+			cairo_set_line_cap( cr, CAIRO_LINE_CAP_SQUARE );
+			cairo_move_to( cr, 0, y );
+			cairo_line_to( cr, w -1, y );
+			cairo_stroke( cr );
 		}
 	}
-	g_object_unref( fg1 );
 }
 
 typedef struct
@@ -142,33 +152,43 @@ typedef struct
 	gint y;
 } point;
 
-void draw_graph_grid( CPUGraph *base, GtkWidget *da, gint w, gint h )
+void draw_graph_grid( CPUGraph *base, cairo_t *cr, gint w, gint h )
 {
 	gint x, y;
 	gint usage;
 	point last, current;
 	last.x = 0;
 	last.y = h;
-	GdkGC *fg1 = gdk_gc_new( da->window );
 
-	gdk_gc_set_rgb_fg_color( fg1, &base->colors[1] );
+	gdk_cairo_set_source_rgba( cr, &base->colors[1] );
+	cairo_set_line_cap( cr, CAIRO_LINE_CAP_SQUARE );
+	cairo_set_line_width( cr, 1 );
+
 	for( x = 0; x * 6 < w; x++ )
 	{
-		gdk_draw_line( da->window, fg1, x*6, 0, x*6, h-1 );
+		/* draw line */
+		cairo_move_to( cr, x * 6 + 0.5, 0.5 );
+		cairo_line_to( cr, x * 6 + 0.5, h - 1 + 0.5 );
+		cairo_stroke( cr );
 	}
 	for( y = 0; y * 4 < h; y++ )
 	{
-		gdk_draw_line( da->window, fg1, 0, y*4, w-1, y*4 );
+		/* draw line */
+		cairo_move_to( cr, 0.5, y * 4 + 0.5 );
+		cairo_line_to( cr, w - 1  + 0.5, y * 4 + 0.5 );
+		cairo_stroke( cr );
 	}
 
-	gdk_gc_set_rgb_fg_color( fg1, &base->colors[2] );
+	gdk_cairo_set_source_rgba( cr, &base->colors[2] );
 	for( x = 0; x < w; x++ )
 	{
 		usage = h * base->history[w - 1- x] / CPU_SCALE;
 		current.x = x;
 		current.y = h - usage;
-		gdk_draw_line( da->window, fg1, current.x, current.y, last.x, last.y );
+		/* draw line */
+		cairo_move_to( cr, current.x + 0.5, current.y + 0.5 );
+		cairo_line_to( cr, last.x + 0.5, last.y + 0.5 );
+		cairo_stroke( cr );
 		last = current;
 	}
-	g_object_unref( fg1 );
 }
