@@ -26,9 +26,9 @@
 
 #include "cpu.h"
 
-void draw_graph_normal( CPUGraph *base, GtkWidget *da, gint w, gint h );
-void draw_graph_LED( CPUGraph *base, GtkWidget *da, gint w, gint h );
-void draw_graph_no_history( CPUGraph *base, GtkWidget *da, gint w, gint h );
-void draw_graph_grid( CPUGraph *base, GtkWidget *da, gint w, gint h );
+void draw_graph_normal( CPUGraph *base, cairo_t *cr, gint w, gint h );
+void draw_graph_LED( CPUGraph *base, cairo_t *cr, gint w, gint h );
+void draw_graph_no_history( CPUGraph *base, cairo_t *cr, gint w, gint h );
+void draw_graph_grid( CPUGraph *base, cairo_t *cr, gint w, gint h );
 
 #endif /* !_XFCE_MODE_H_ */
